<?php

/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * The Codes Object is a singleton that contains all of the codes loaded from the codes table
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_codes {

   private $_codes;
   private $_codes_dao;

   
/**
 * This function will return a singleton instance of the sys_codes object
 * 
 * @return a singleton instance of sys_codes
 * 
 */   
   static public function getInstance(){
      static $codes_instance;
      if (!isset($codes_instance)) {
         $codes_instance = new sys_codes();
      }
      return $codes_instance;
   }

/**
 * This is the private constructor for the sys_codes object
 * 
 * It will connect to the database and read in all of the codes which are then available to other modules
 * 
 */   
   private function __construct() {
      $connObject = sys_connect_db::getInstance();
      $this->_conn = $connObject->getConnection();
      $this->_codes_dao = new sys_codes_dao();
      $codes = $this->_codes_dao->loadAllCodes();
      foreach ($codes as $code) {
         $this->_codes[$code->getCodeType()][$code->getCodeValue()] = $code;
      }
   }

/**
 * This routine will retrieve the codes from the sys_codes object in an array of sys_code_bean objects
 * 
 * @param string $codeType specifies which codes to return
 * @return array containing either all codes or those indicated by the $codeType
 */   
   public function getCodes($codeType = null) {
      if ($codeType == null) {
         return $this->_codes;
      }
      if (key_exists($codeType,$this->_codes)) {
         return $this->_codes[$codeType];
      }
      return array();
   }
   
/**
 * This routine will retrieve the codes in an array of sys_flag_bean objects
 * 
 * @param string $codeType specifies which flags to return
 * @return array containing either flags for all codes or those indicated by the $codeType
 */   
   public function getFlags($codeType) {
      $codes = $this->getCodes($codeType);
      $flags = array();
      foreach ($codes as $code) {
         $flag = new sys_flag_bean();
         $name = $code->getCodeValue();
         $desc = $code->getDescription();
         $parm = $code->getCodeParm1();
         $flag->setName($name);
         $flag->setDescription($desc);
         if ($parm == 'Y') {
            $flag->setFieldValue('Y');
         } else {
            $flag->setFieldValue('N');
         }
         $flag->setSeq($code->getSeq());
         $flag->setParm1($code->getCodeParm1());
         $flag->setParm2($code->getCodeParm2());
         $flag->setParm3($code->getCodeParm3());
         $flags [] = $flag;
         
      }
      return $flags; 
   }
   
/**
 * This routine will format a criterion bean with criteria from the named code
 * 
 * @param string $fieldName is initialized in the criterion bean
 * @param string $codeName used to get the appropriate flags
 * @param string $codeType is initialized in the criterion bean
 * @return sys_criterion_bean for this code
 */   
   public function getCriterion($fieldName, $codeName, $codeType) {
       $criterion = new sys_criterion_bean();
       $criterion->setFieldName($fieldName);
       $criterion->setCodeName($codeName);
       $criterion->setFlags($this->getFlags($codeName));
       $criterion->setCodeType($codeType);
       return $criterion;
   }   
   

/**
 * This routine will retrieve the codes in an associative array
 * 
 * @param string $codeType specifies which codes to return
 * @return array containing either all codes or those indicated by the $codeType
 */   
   public function getCodeArray($codeType) {
      $codes = $this->getCodes($codeType);
      foreach ($codes as $code) {
         if ($code->getCodeType() == $codeType) {
              $id        = $code->getId();
              $codeValue = $code->getCodeValue();
              $desc      = $code->getDescription();
              $parm1     = $code->getCodeParm1();
              $parm2     = $code->getCodeParm2();
              $parm3     = $code->getCodeParm3();
              $ret_codes []  = array ("code" => "$codeValue",
                                 "codeParm1" => "$parm1",
                                 "codeParm2" => "$parm2",
                                 "codeParm3" => "$parm3",
                               "description" => "$desc",
                                        "id" => "$id");
         }
      }
      return $ret_codes;
   }
   
   
/**
 * This routine will retrieve a single code based on its type and value
 * for example: states/OR would return the code for Oregon
 * 
 * @param string $codeType specifies the type of the code to return
 * @param string $codeValue specifies the value of the code to return
 * @return sys_code_bean object containing the specified code
 */   
   public function getCode($codeType,$codeValue) {
   	  $codes = $this->getCodes($codeType);
   	  if (key_exists($codeValue, $codes)) {
        return $codes[$codeValue];
   	  } else {
   	  	$code = new sys_code_bean();
   	  	return $code;
   	  }   
   }
}