<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 *
 * The Configuration Object is a singleton that contains configuration
 * options and is available anywhere in the code via the getInstance static
 * method.
 *
 * The data is loaded from the config.ini file located in the config directory
 * which is specified in the main index.php file.
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 *
 */
class sys_config {



	private $_config;
	private $_CONFIG_FILE =	"/config/config.ini";

	/**
	 * This is the private constructor of the sys_config object
	 *
	 */
	private function __construct() {
		$this->_config = parse_ini_file(CONFIG_PATH.$this->_CONFIG_FILE);
		if (!$this->hasEntry('useUnixTime')) {
			$this->setEntry('useUnixTime', 'N');
		}
	}

	/**
	 * This function will return a singleton instance of the sys_config object
	 *
	 * @param array $config - contains an associative array of the configuration options
	 * @return a singleton instance of sys_config
	 */
	static public function getInstance($config = null){
		static $config_instance;
		if (!isset($config_instance)) {
			$config_instance = new sys_config($config);
		}
		return $config_instance;
	}


	/**
	 * This function will save the ini data back to the file
	 *
	 * Note: this will overwrite any comments in the file
	 * @return true if successfull
	 */
	public function save() {

		$content = ";; this is the application configuration ini file \n\n";
		foreach ($this->_config as $key=>$val) {
			if ($key != 'id') {
				$content .= str_pad($key, 15)." = ".$val."\n";
			}
		}

		if (!$handle = fopen(CONFIG_PATH.$this->_CONFIG_FILE, 'w')) {
			return false;
		}
		if (!fwrite($handle, $content)) {
			return false;
		}
		fclose($handle);
		return true;
	}


	/**
	 * This function will retrieve an entry from the config object
	 *
	 * @param string $entry  the entry to return
	 * @return string the value of the $entry specified
	 */
	public function getEntry ($entry, $default = '') {
		if ($this->hasEntry($entry)) {
			return $this->_config[$entry];
		} else {
			return $default;
		}
	}

	/**
	 * This function will see if there is a particular entry in the config object
	 *
	 * @param string $entry  the entry to look for
	 * @return bool true if found else false
	 */
	public function hasEntry ($entry) {
		return isset($this->_config[$entry]);
	}

	/**
	 * This function will retrieve the entire sys_config associative array
	 *
	 * @return array the entire sys_config associative array
	 */
	public function getConfig () {
		return $this->_config;
	}

	/**
	 * This function set a configuration option
	 *
	 * @param string $entry the entry to set
	 * @param string $value to value to set it to
	 */
	public function setEntry($entry, $value) {
		$this->_config[$entry] = $value;
	}

	/**
	 * This function will retrieve the entire sys_config associative array as a sys_config_bean
	 *
	 * @return sys_config_bean the entire sys_config associative array in a bean
	 */
	public function getConfigBean() {
		$bean = new sys_config_bean();
		foreach ($this->_config as $key => $val) {
			$bean->set($key,$val);
		}
		return $bean;
	}

	/**
	 * This function will update the sys_config associative array with the contents of the passed bean
	 *
	 * @param sys_config_bean bean the bean that contains the updates
	 */
	public function setConfigBean($bean) {
		$fields = $bean->getBeanFields();
		foreach ($fields as $key => $val) {
			$this->setEntry($key,$val);
		}
	}



}