<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * This is the main front controller module for the framework.  It has two main
 * functions.  1) process post actions and 2) process get (navigation) requests. 
 * 
 * Post requests are considered post actions.  They will update something
 * in the system. Post actions may have logic to determine the validity of the 
 * request and if invalid respond accordingly, or if everything is jake, update 
 * the appropriate resources.  Once a post action is complete, it returns to the 
 * controller with an indication of the next Get (navigation) action.
 * 
 * A redirect is always done to the Get url in order to replace the post transaction 
 * in the browser's history.  This prevents the annoying repost issues when you hit 
 * the refresh or back button.
 * 
 * Get requests are considered navigation actions.  They will result in a screen 
 * being displayed.  The navigation actions are placed in a loop however.  This is so
 * that if the navigation action gets so far into the processing and discovers that 
 * another navigation action would be more appropriate, it can return with the url
 * for that navigation action.  This process is repeated until an action returns with
 * "done".
 * 
 * This module also ensures that if required the user is logged on and/or is an 
 * administrator.
 *
 * @package     hardhat_framework
 * @subpackage  classes
 * @author      Rod Johnson
 * 
 */
class sys_front_controller {

    private $_request;
    private $_registry;
    private $_config;

    public function __construct(){
        $this->_registry = sys_registry::getInstance();
        $this->_request = sys_request::getInstance();
        $this->_config = sys_config::getInstance();
        $this->_commands = sys_commands::getInstance();
    }

    public function processRequest() {
        if ($this->_request->isPost()) {
            $this->processPost();
            return;
        } elseif ($this->_request->isGet())  {
            $this->processGet();
            return;
        } else {
            echo "unsupported request type = ".$this->_request->getRequestType();
            return;
        }
    }
    private function processPost() {
        $login = $this->_registry->getEntry('core', 'login');
        $cmd = $this->_request->getCommand();
        $actionBean = $this->_request->getCommandBean();
        if ($actionBean == null) {
            $feedback->setMsg("Undefined Action: $cmd", 'E');
            $cmd = 'error';
            $urlString = makeRelativeURL($cmd);
            $this->_registry->setEntry('core','feedback', $feedback);
            header("Location: $urlString");
            return;
        }
        $login = $this->_registry->getEntry('core', 'login');
        $feedback = $this->_registry->getEntry('core', 'feedback');
        $admin = $login->getAdmin();
        $loggedIn = $login->getLoggedIn();
        $adminReq = $actionBean->getAdminReq();
        $loggedInReq = $actionBean->getLoggedInReq();
        if (($actionBean->getLoggedInReq() == 'Y') && ($login->getLoggedIn() != 'Y')) {
            $feedback->setMsg("Must be logged in for this function", 'E');
            $cmd = 'home';
            $urlString = makeRelativeURL($cmd);
            $this->_registry->setEntry('core','feedback', $feedback);
            header("Location: $urlString");
            return;
        }
        if (!$this->authorized($actionBean,$login,$feedback)){
            $cmd = 'home';
            $urlString = makeRelativeURL($cmd);
            $this->_registry->setEntry('core','feedback', $feedback);
            header("Location: $urlString");
            return;
        }
        if ($login->getUserStatus() != 'A') {
            $feedback->setMsg("UserId is Suspended or Inactive", 'E');
            $cmd = 'logoff';
            $urlString = makeRelativeURL($cmd);
            $this->_registry->setEntry('core','feedback', $feedback);
            header("Location: $urlString");
            return;
        }
        $actionClass = $actionBean->getActionClass();

        if ($actionClass == null) {
            echo 'action class is null';
        }
        if ($actionClass != '' && class_exists($actionClass)) {
            $actionInstance = new $actionClass();
        } else {
            $feedback->setMsg
            ('System Error - Undefined Post Action '.$actionClass, 'E');
            $cmd = 'error';
            $urlString = makeRelativeURL($cmd);
            $this->_registry->setEntry('core','feedback', $feedback);
            header("Location: $urlString");
            return;
        }

        $this->_registry->setEntry('core', 'action', $actionClass);
        
        $cmd = $actionInstance->process($actionBean,$feedback,$login);
        $urlString = makeRelativeURL($cmd);

        $this->_registry->setEntry('core','feedback', $feedback);
        $this->_registry->setEntry('core', 'postVars', $this->_request->getPostVars());
        $this->_registry->setEntry('core', 'login', $login);
        header("Location: $urlString");
        return;
    }

    private function processGet() {
    	$feedback = $this->_registry->getEntry('core','feedback');
        $postVars = $this->_registry->getEntry('core','postVars');
        $this->_request->setPostVars($postVars);
        $login = $this->_registry->getEntry('core', 'login');
        $cmd = $this->_request->getCommand();
        while ($cmd != 'done') {
            if (!$this->_request->parseUrl($cmd)) {
            	if ($this->_request->getMessage() != '') {
                	$feedback->setMsg($this->_request->getMessage(), 'E');
            	}
            }
            $nav = $this->_request->getCommandBean();
            if ($nav == null) {
                $feedback->setMsg("Invalid command $cmd", 'E');
                $cmd = 'error';
                continue;
            }
            $obj = $nav->getActionClass();
            if ($obj != '' && class_exists($obj)) {
                $act = new $obj();
            } else {
                $feedback->setMsg
                ('System Error - Undefined Navigation Action '.$obj, 'E');
                $cmd = 'error';
                continue;
            }
            if (!is_object($login)) {
            	$login = new sys_user_bean();
            }	
           	$admin = $login->getAdmin();
           	$loggedIn = $login->getLoggedIn();
            $adminReq = $nav->getAdminReq();
            $loggedInReq = $nav->getLoggedInReq();
            if (!$this->authorized($nav,$login,$feedback)) {
                $cmd = 'error';
                continue;
            }
            if (($nav->getLoggedInReq() == 'Y') && ($login->getLoggedIn() != 'Y')) {
                $feedback->setMsg("Must be logged in for this function", 'E');
                $postLoginAction = $this->_request;
                $this->_registry->setEntry('core','postLoginAction', $postLoginAction);
                $cmd = 'login';
                if ($nav->getLoginPath() != '') {
                	$cmd = $nav->getLoginPath();
                }
                continue;
            }
            if (($nav->getLoggedInReq() == 'Y') && ($login->getUserStatus() != 'A')) {
                $feedback->setMsg("UserId is Suspended or Inactive - Contact Customer Support", 'E');
                $cmd = 'home';
                continue;
            }
            $cmd = $act->perform($nav,$feedback,$login);
            if ($cmd == 'done') continue;
        }
        $this->_registry->unsetEntry('core', 'feedback');
        $this->_registry->unsetEntry('core', 'postVars');
        $this->_registry->unsetEntry('sys', 'getVars');

    }
    
    private function authorized($action,$login,$feedback){
        if (($action->getAdminReq() == 'Y') && ($login->getAdmin() != 'Y')) {
            $feedback->setMsg("Not Authorized", 'E');
            return false;
        }
        return true;
    }

}