<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 *
 * This is the base class for PDO dao's in the Hard Hat Framework
 *
 * This class is meant to be extended in the application to database access.  The constructor specifies
 * a table, view, and a bean.  The view is used for reading data and returns a bean or array of beans.
 * The table is used for insert, update and delete processing.
 *
 *
 * @package     hardhat_framework
 * @subpackage  controller
 * @author      Rod Johnson
 *
 */
class sys_base_pdo_dao
{
	protected $_conn;
	private $_tableName;
	private $_viewName;
	private $_beanName;
	private $_schema;
	private $_viewSchema;
	private $_fieldList;
	private $_viewFieldList;
	private $_daoFields;
	private $_viewDaoFields;
	private $_identity;
	private $_tableExists;
	protected $_config;

	// ---------------------------------------------------- constructor
	public function __construct($tableName, $beanName, $viewName = null, $identity = 'id'){
		$this->_config = sys_config::getInstance();
		if ($viewName == null) {
			$viewName = $tableName;
		}
		$this->_identity = $identity;
		$connObject = sys_connect_pdo_db::getInstance();
		$this->_conn = $connObject->getConnection();
		$this->_viewSchema = array();
		$this->_viewFieldList = '';
		$this->_viewDaoFields = array();
		/*
		 *      load the view fieldList for select processing
		*/
		$query  = "SHOW COLUMNS FROM $viewName";
		$result = $this->_conn->query($query);
		if ($result->rowCount() == 0) {
			$this->_tableExists = false;
			return false;
		}
		$this->_tableExists = true;
		while ($row = $result->fetch(PDO::FETCH_ASSOC)) {
			$fieldInfo = $row;
			$type = $row['Type'];
			$dataType = snip($type,'(');
			$length = snip($type,')');
			$fieldInfo ['DataType'] = $dataType;
			$fieldInfo ['Length'] = $length;
			$fieldName = $row['Field'];
			$this->_viewSchema [$fieldName] = $fieldInfo;
			$this->_viewDaoFields [] = $row['Field'];
			$this->_viewFieldList .= '`'.$row['Field'].'`, ';
		}
		$this->_viewFieldList = substr($this->_viewFieldList,0,-2);

		/*
		 *   load the table daoFields for insert update processing
		*/
		$this->_schema = array();
		$this->_fieldList = '';
		$this->_daoFields = array();
		$query  = "SHOW COLUMNS FROM $tableName";
		$result = $this->_conn->query($query);
		while ($row = $result->fetch(PDO::FETCH_ASSOC)) {
			$fieldInfo = $row;
			$type = $row['Type'];
			$dataType = snip($type,'(');
			$length = snip($type,')');
			$fieldInfo ['DataType'] = $dataType;
			$fieldInfo ['Length'] = $length;
			$fieldName = $row['Field'];
			$this->_daoFields [] = $row['Field'];
			$this->_schema [$fieldName] = $fieldInfo;
			$this->_viewDaoFields [] = $row['Field'];
			$this->_fieldList .= '`'.$row['Field'].'`, ';
		}
		$this->_fieldList = substr($this->_fieldList,0,-2);
		$this->_beanName = $beanName;
		$this->_tableName = $tableName;
		$this->_viewName = $viewName;
	}

	// ---------------------------------------------------- getters
	public function getEmptyBean(){
		$beanName = $this->getBeanName();
		$bean = new $beanName;
		$fields = $this->getSchema();
		foreach ($fields as $key => $val) {
			$bean->set($key,'');
		}
		return $bean;
	}
	public function getInitializedBean(){
		$beanName = $this->getBeanName();
		$bean = new $beanName;
		$fields = $this->getSchema();
		foreach ($fields as $key => $val) {
			if (!$bean->fieldExists($key)) {
				$bean->set($key,'');
			}
		}
		return $bean;
	}
	public function getTableName(){
		return $this->_tableName;
	}
	public function getViewName(){
		return $this->_viewName;
	}
	public function getBeanName(){
		return $this->_beanName;
	}
	public function getSchema(){
		return $this->_schema;
	}
	public function getViewSchema(){
		return $this->_viewSchema;
	}
	public function getFieldList(){
		return $this->_fieldList;
	}
	public function getViewFieldList(){
		return $this->_viewFieldList;
	}
	public function getDaoFields(){
		return $this->_daoFields;
	}
	public function getViewDaoFields(){
		return $this->_viewDaoFields;
	}
	public function getTableExists(){
		return $this->_tableExists;
	}
	public function getFieldInfo($fieldName) {
		$tableName = $this->_tableName;
		$fields = $this->getViewDaoFields();
		if (!in_array($fieldName, $fields)) {
			$className = get_class($this);
			throw new Exception("Class $className - field ($fieldName) not found in table ($tableName).");
			exit();
		}
		$schema = $this->getViewSchema();
		return $schema [$fieldName];
	}

	protected function getBean($row){

		$beanName = $this->getBeanName();
		$bean = new $beanName;
		foreach ($row as $field => $value) {
			$fieldInfo = $this->getFieldInfo($field);
			$dataType = $fieldInfo ['DataType'];
			if ($this->_config->getEntry('useUnixTime') == 'Y') {
				if ($dataType == 'date' or $dataType == 'datetime') {
					$value = strtotime($value);
				}
			} else
				if ($dataType == 'date') {
				$value = strtodate($value);
			}
			$setMethod = "set$field";
			$bean->$setMethod($value);
		}
		$bean = $this->buildBean($bean);
		return $bean;
	}

	protected function buildBean($bean) {
		return $bean;
	}

	public function prep($data, $field){
		$fieldInfo = $this->getFieldInfo($field);
		$dataType = $fieldInfo ['DataType'];
		$dataLength = $fieldInfo ['Length'];
		if (is_object($data)) {
			return fmtString(serialize($data));
		} else {
			if (is_array($data)) {
				return fmtString(serialize($data));
			}
			switch ($dataType) {
				case 'tinyint'  :
				case 'smallint'  :
				case 'int'      : return fmtInt($data);
				case 'double'   : return is_numeric($data) ? $data : 'null';
				case 'float'    : return is_numeric($data) ? $data : 'null';
				case 'text'     :
				case 'longtext' :
				case 'char'     :
				case 'enum'     :
				case 'time'     :	
				case 'varchar'  : return $data;
				case 'decimal'  : return $data ? $data : 0;
				case 'date'     :
				case 'datetime'	:  {
					if (!isset($data)){
						return null;
					}
					if (!is_int($data)) {
						return null;
					}
					$r = date('c',$data);
					$r = substr($r,0,strlen($r)-7);
					return $r;
				} 
				default         : 
					return fmtString(serialize($data));
			}
		}
		//*/
	}
	protected function runQuery($query){
		$result = $this->_conn->query($query);
		if ($row = $result->fetch(PDO::FETCH_ASSOC)) {
			return $row;
		}
		return null;
	}
	 

	public function executeQuery($query){
		$result = $this->_conn->query($query);
		$result = $result->fetchAll(PDO::FETCH_ASSOC);
		return $result;
	}

	protected function getFlags($fieldName){
		$fieldInfo = $this->getFieldInfo($fieldName);
		$tableName = $this->getTableName();
		$beans = array();
		$query  = "select distinct $fieldName from $tableName";
		$result = $this->_conn->query($query);
		while ($row = $result->fetch(PDO::FETCH_ASSOC)) {
			$bean = new sys_flag_bean();
			$bean->set('name',$row[$fieldName]);
			$beans [] = $bean;
		}
		return $beans;
	}

	protected function loadBeanById($id){
		$beanName = $this->getBeanName();
		$bean = new $beanName;
		$flist = $this->getFieldList();
		$tableName = $this->getViewName();
		$id = fmtInt($id);
		$query  = "select $flist from $tableName where ";
		$query .= $this->_identity." = $id";
		$result = $this->_conn->query($query);
		if ($row = $result->fetch(PDO::FETCH_ASSOC)) {
			$bean = $this->getBean($row);
		}
		return $bean;
	}

	public function loadBean($criteria, $orderBy = null){
		$values = array();
		if (is_array($criteria)) {
			$values = $criteria['values'];
			$where = $criteria['where'];
		} else {
			$where = $criteria;
		}
		$bean = null;
		$flist = $this->getViewFieldList();
		$tableName = $this->getViewName();
		$query  = "select $flist from $tableName where ";
		$query .= "$where ";
		if ($orderBy != null) {
			$query .= "order by $orderBy";
		}
		$result = $this->_conn->prepare($query);
		$result->execute($values);
		if ($row = $result->fetch(PDO::FETCH_ASSOC)) {
					$bean = $this->getBean($row);
		}
		return $bean;
	}
	public function loadBeans($criteria, $orderBy = null, $limit = null){
		$values = array();
		if (is_array($criteria)) {
			$values = $criteria['values'];
			$where = $criteria['where'];
		} else {
			$where = $criteria;
		}
		$flist = $this->getViewFieldList();
		$tableName = $this->getViewName();
		$query  = "select $flist from $tableName where ";
		$query .= "$where ";
		if ($orderBy != null) {
			$query .= " order by $orderBy ";
		}
		if ($limit != null) {
			$query .= " Limit $limit ";
		}
		$result = $this->_conn->prepare($query);
		$result->execute($values);
		$beans = array();
		while ($row = $result->fetch(PDO::FETCH_ASSOC)) {
			$bean = $this->getBean($row);
			$beans [] = $bean;
		}
		return $beans;
	}
	protected function saveBean($bean,$user){
		$id = $bean->getId();
		if (!isset($id)) {
			return $this->insertBean($bean,$user);
		}
		if ($id == 0) {
			return $this->insertBean($bean,$user);
		}
		$temp = $this->loadBeanById($bean->getId());
		if ($temp == null) {
			return $this->insertBean($bean,$user);
		} else {
			return $this->updateBean($bean,$user);
		}
	}
	 
	protected function insertBean(sys_base_bean $bean, $user = null){
		if ($user == null) {
			$user = new sys_user_bean();
		}
		$fields = $this->getDaoFields();
		$tableName = $this->getTableName();
		$flist = '';
		$values = array();
		$vfields = array();
		foreach ($fields as $field) {
			$fieldInfo = $this->getFieldInfo($field);
			$dataType = $fieldInfo ['DataType'];
			if ($dataType != 'timestamp') {
				$lowerfield = strtolower($field);
				if ($bean->fieldExists($field) && $field != $this->_identity &&
						$lowerfield != 'updatedate' &&
						$lowerfield != 'createdate' &&
						$lowerfield != 'createby' &&
						$lowerfield != 'updateby') {
					$flist .= $field.", ";
					$value = $bean->get($field);
					$value = $this->prep($value,$field);
					$vfield = ':f_'.$field;
					$values [$vfield] = $value;
					$vfields [] = $vfield;
 					} else {
					$found = false;
					switch (strtolower($field)) {
						case 'updatedate' :
						case 'createdate' : $value = date('Y-m-d H:i:s');
						$found = true;
						break;
						case 'createby'   :
						case 'updateby'   : $value = $user->getId();
						$found = true;
						break;
					}
					if ($found) {
						$flist .= $field.", ";
						$vfield = ':f_'.$field;
						$values [$vfield] = $value;
						$vfields [] = $vfield;
					}
				}
			}
		}
		$flist = substr($flist,0,-2);
		$vfields = implode(',',$vfields);
		$query  = "INSERT INTO $tableName ($flist) values ($vfields)";
		$result = $this->_conn->prepare($query);
		$result->execute($values);
		return $this->_conn->lastInsertId();
	}
	
	protected function updateBean(sys_base_bean $bean, sys_user_bean $user){
		$fields = $this->getDaoFields();
		$tableName = $this->getTableName();
		$flist = $this->getFieldList();
		$query  = "UPDATE $tableName SET ";
		foreach ($fields as $field) {
			$fieldInfo = $this->getFieldInfo($field);
			$dataType = $fieldInfo ['DataType'];
			if ($dataType != 'timestamp') {
				$lowerfield = strtolower($field);
				if (($field != $this->_identity) && $bean->fieldExists($field) &&
						$lowerfield != 'updatedate' &&
						$lowerfield != 'createdate' &&
						$lowerfield != 'createby' &&
						$lowerfield != 'updateby') {
					$value = $bean->get($field);
					$value = $this->prep($value,$field);
					$vfield = ':f_'.$field;
					$query .= "`$field` = $vfield, ";
					$values [$vfield] = $value;
				} else {
					$found = false;
					switch (strtolower($field)) {
						case 'updatedate' : $value = date('Y-m-d H:i:s');
						$found = true;
						break;
						case 'updateby'   : $value = fmtIntDefault($user->getId(),0);
						$found = true;
						break;
					}
					if ($found) {
						$vfield = ':f_'.$field;
						$query .= "`$field` = $vfield, ";
						$values [$vfield] = $value;
					}
				}
			}
		}
		$query = substr($query,0,-2);
		$value = fmtInt($bean->getId());
		$query .= " where $this->_identity = :f_id";
		$values [':f_id'] = $value;
		$result = $this->_conn->prepare($query);
		$result->execute($values);
		return $result->rowCount();
	}

	protected function deleteBeans($criteria){
		$values = array();
		if (is_array($criteria)) {
			$values = $criteria['values'];
			$where = $criteria['where'];
		} else {
			$where = $criteria;
		}
		$tableName = $this->getTableName();
		$query  = "DELETE from $tableName where $where";
		$result = $this->_conn->prepare($query);
		$result->execute($values);;
		return $result->rowCount();
	}

	protected function deleteBean($id){
		$tableName = $this->getTableName();
		$query  = "DELETE from $tableName where $this->_identity = :f_id";
		$result = $this->_conn->prepare($query);
		$result->execute(array(':f_id' => $id));
		return $result->rowCount();
	}

}
