<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */

/**
 * 
 * This class is the base class for any bean class.  It encapsulates the data in the beanFields array.
 * This class may be extended and used to represent model data for the application.  BeanFields are
 * accessed using getters and setters, which are interpreted via the _call magic function. The advantage is
 * that all access to the data is through getters and setters, which do not need to be explicitly specified,
 * but which can be overriden in the classes that extend this class.
 *
 * This class may be extended and used to receive data using extended versions of the sys_base_dao class.
 *
 * Getters and Setters of this class may use formatting parameters to format the data in various ways.  This
 * is particularly useful in the templates.
 *
 * For example:
 * {$user->getBirthDate('shortdate')}
 *
 * or
 *
 * <select name="gender" id="gender">
 *  <option value="" >please specify gender</option>
 *  <option value="M" {$user->getGender('checked', 'M')}>Male</option>
 *  <option value="F" {$user->getGender('checked', 'F')}>Female</option>
 * </select>
 *
 * @package     hardhat_framework
 * @subpackage  database
 * @author      Rod Johnson
 * 
 */
class sys_base_bean {

    protected $_beanFields;

    // ---------------------------------------------------- constructor
    public function __construct(){
        $this->_beanFields = array();
        $this->_beanFields['id'] = null;
    }

    public function __get($var) {
        $className = get_class($this);
        throw new Exception("invalid attempt to get property $var in class $className probably missing () on a method call");
    }

    public function __set($var, $val){
        $className = get_class($this);
        throw new Exception("invalid attempt to set property $var in class $className probably missing () on a method call");
    }

    public function __call($function, $args){
        $className = get_class($this);
        $cmd = substr($function,0,3);
        $propertyName = $this->uncapitalize(substr($function,3));
        if (!is_array($this->_beanFields)) {
            $this->_beanFields = array();
            $this->_beanFields['id'] = null;
        }

        if ($cmd == 'get'){
            if (!key_exists($propertyName,$this->_beanFields)) {
                return '';
            }
            $property = $this->_beanFields[$propertyName];
            $subcmd = null;
            $parm2 = null;
            if (is_array($args)) {
                if (count($args) > 0) {
                    $subcmd = strtolower($args[0]);
                    if (count($args) > 1) {
                        $parm2 = (count($args)> 1) ? $args[1] : '';
                    }
                }
            }
            return $this->convert($property, $subcmd, $parm2);
        } else
        if ($cmd == 'set'){
            if (is_array($args)) {
                if (count($args) > 0) {
                    if (count($args) > 1) {
                        $this->_beanFields[$propertyName] = $this->setValue($args[0],$args[1]);
                    } else {
                        $this->_beanFields[$propertyName] = $args[0];
                    }
                }
            }
            return;
        } else
        if ($cmd == 'cnt'){
            if (is_array($this->_beanFields[$propertyName])) {
                return count($this->_beanFields[$propertyName]);
            }
        } else {
            throw new Exception("Class $className - undefined method $function.");
        }
    }
    static public function setValue($value, $subcmd = ''){
        if ($subcmd == '') {
            return $value;
        }
        switch ($subcmd) {
            case 'datetime'  : {
//                $da = explode('/',$value);
 //               $time = mktime(0,0,0,$da[0],$da[1],$da[2]);
                $datetime = date('c',$value);
                $datetime = substr($datetime,0,strlen($datetime)-7);
                return "'$datetime'";
            }
        }
    }
     
    static public function convert($property, $subcmd = null, $parm2 = null){
        if ($subcmd == null) {
            return $property;
        }
        switch ($subcmd) {
            case 'string'  :
                return $property == null ? '' : $property;
            case 'dbsafe'  :
                return mysql_real_escape_string($property);
            case 'websafe'  :
                return nl2br(strip_tags(htmlentities($property)));
                case 'websafenl'  :
                return strip_tags(htmlentities($property));
            case 'selected' :
                return $property == $parm2 ? 'selected="selected"' : '';
            case 'checked'  :
                return $property == $parm2 ? 'checked="checked"' : '';
            case 'yesno'    :
                if ($property == 'Y'){return 'Yes';}
                if ($property == 'N') {return 'No';}
                return 'Not Specified';
            case 'yes'      :
                return $property == 'Y' ? 'Yes' : '';
            case 'max'  :
                return $property < $parm2 ? $property : $parm2;
            case 'datephp'  :
            	if ($property) {
                return date($parm2,$property);
            	} else{
            		return '';
            	}
            case 'codes'  :
            	$codes = sys_codes::getInstance();
            	return $codes->getCode($parm2,$property)->getDescription();
            case 'money'  :
                return '$' . $property;
            case 'dbdate'   :
                $date = date('c',$property);
                return snip($date,'T');
            case 'shortdatetime' :
                return date('n/d/Y h:i A',strtotime($property));
            case 'shortdate':
                if ($property == null) return '';
                if (trim($property) == '') return '';

                return date('n/d/Y',$property);
            case 'longdate':
                return date('l F j, Y',$property);
            case 'exceldate':
                $date = date('c',$property);
                return snip($date,'T')."T00:00:00.000";
            case 'swapdate':
                $date = strtotime($property);
                if (!$date) {
                    return '';
                }
                return date($parm2, $date);
            default :
                return 'unknown subcmd '.$subcmd;
        }
    }
     
    public function initForRedisplay ($exclusions = array()) {
        if (!is_array($exclusions)) {
            $exclusions = array($exclusions);
        }
        $request = sys_request::getInstance();
        $fields = $this->getBeanFields();
        foreach ($fields as $key => $val) {
            if (!key_exists(strtolower($key),$exclusions)) {
                if ($request->hasPostVar($key)) {
                    $val = $request->getPostVar($key);
                    $this->set($key,$val);
                }
            }
        }
    }
    
    public function fieldExists($field){
        return (key_exists(strtolower($field), $this->_beanFields));
    }

    public function getBeanFields() {
        return $this->_beanFields;
    }

    public function setBeanFields($s) {
        $this->_beanFields = $s;
    }

    public function dumpBeanFields($sep = '<br />') {
        $s = '';
        foreach ($this->_beanFields as $key => $value){
            if (is_array($value)) {
                $s .= $key." [array] $sep";
                foreach ($value as $k => $v) {
                    $s .= "[$k] = $v $sep";
                }
            } else {
                $s .= "$key = $value $sep";
            }
        }
        return $s;
    }

    public function set($property,$value) {
        $prop = $this->uncapitalize($property);
        $this->_beanFields[$prop] = $value;
    }

    public function get($property) {
        $prop = $this->uncapitalize($property);
        if (key_exists($prop,$this->_beanFields)) {
            return $this->_beanFields[$prop];
        }
        return null;
    }

    protected function capitalize($s){
        return strtolower($s);
        //make bean fields case insensitive
        //return strtoupper(substr($s,0,1)).substr($s,1);
    }

    protected function uncapitalize($s){
        return strtolower($s);
        //make bean fields case insensitive
        //return strtolower(substr($s,0,1)).substr($s,1);
    }

}