<?php
/**
 * Hardhat Framework
 *
 * An open source application development framework for PHP 5.2 or newer
 *
 * @package     hardhat_framework
 * @author		Rod Johnson
 * @copyright	Copyright (c) 2008 - 2012, Spellman Systems LLC
 * @license		http://hardhatframework.com/doc/license
 * @link		http://hardhatframework.com
 */


/**
 * This is the autoload function for the Hard Hat Framework
 *
 * This routine uses the php magic function __autoload to load the appropriate php module
 * with the desired class.  The class name gives us some clues about where the module is
 * going to be found.  First, if the class name starts with sys, it is part of the
 * hardhat framework and will be found in one of those directories.
 * 
 * If not, the module name is designated by the first node.  The next level may contain the
 * type, such as nav, post, ajax, db, or model, which may be overridden by the last node
 * which could be action, bean, dao or delegate for a post action, model or database module
 * respectively
 * 
 * Note: the module may be found in the modules directory within the application or it may be
 * in a separate place designated by the MODULE_PATH.  This routine will look first in the
 * application and then in the MODULE_PATH allowing the application to override the Module library.
 * 
 * @param string $classname The name of the class the system did not find already loaded.
 * 
 * @return void
 * 
 */
function hh_sys_autoload($classname)
{
    $classParts = explode('_', $classname);
    if ($classParts[0] === 'sys') {
        autoloadSystem($classname);
        return;
    }
    if ($classParts[0] === 'hh') {
        autoloadExtention($classname);
        return;
    }
    $module = array_shift($classParts);
    $type   = array_shift($classParts);
    $last   = array_pop($classParts);
    if ($last == 'bean') $type = 'model';
    if ($last == 'dao')  $type = 'db';
    if ($last == 'delegate')  $type = 'db';
    if ($last == 'class')  $type = 'class';
    $stub   = "/modules/$module";

    // first check for the resource in the application itself, then check the optional MODULE_PATH
    // the application can override anything in the modules library.  
    if ($type == 'model') {
    	if (file_exists(APPLICATION_PATH."$stub/model/{$classname}.php")) {
	        require_once APPLICATION_PATH."$stub/model/{$classname}.php";
    	    return;
    	} else {    
	        if (file_exists(MODULE_PATH."/$module/model/{$classname}.php")) {
		        require_once MODULE_PATH."/$module/model/{$classname}.php";
		        return;
	        } else{
	        	hhClassError($classname);
	        	return;
	        }
    	}    
    }
    if ($type == 'post') {
    	if (file_exists(APPLICATION_PATH."$stub/controller/post_action/{$classname}.php")) {
    		require_once APPLICATION_PATH."$stub/controller/post_action/{$classname}.php";
        	return;
    	} else {	
	        if (file_exists(MODULE_PATH."/$module/controller/post_action/{$classname}.php")) {
		        require_once MODULE_PATH."/$module/controller/post_action/{$classname}.php";
		        return;
        	} else{
       			hhClassError($classname);
       			return;
       		}
    	}
    }
    if ($type == 'nav') {
    	if (file_exists(APPLICATION_PATH."$stub/controller/navigation_action/{$classname}.php")) {
    		require_once APPLICATION_PATH."$stub/controller/navigation_action/{$classname}.php";
        	return;
    	} else {
    		if (file_exists(MODULE_PATH."/$module/controller/navigation_action/{$classname}.php")) {
	    		require_once MODULE_PATH."/$module/controller/navigation_action/{$classname}.php";
    			return;
   			} else{
   				hhClassError($classname);
   				return;
   			}
    	}	
    }
    if ($type == 'ajax') {
    	if (file_exists(APPLICATION_PATH."$stub/controller/ajax_action/{$classname}.php")) {
    		require_once APPLICATION_PATH."$stub/controller/ajax_action/{$classname}.php";
        	return;
    	} else {
    		if (file_exists(MODULE_PATH."/$module/controller/ajax_action/{$classname}.php")) {
	    		require_once MODULE_PATH."/$module/controller/ajax_action/{$classname}.php";
    			return;
   			} else{
   				hhClassError($classname);
   				return;
   			}
    	}	
    }
    if ($type == 'batch') {
    	if (file_exists(APPLICATION_PATH."$stub/controller/batch_action/{$classname}.php")) {
    		require_once APPLICATION_PATH."$stub/controller/batch_action/{$classname}.php";
        	return;
    	} else {
    		if (file_exists(MODULE_PATH."/$module/controller/batch_action/{$classname}.php")) {
	    		require_once MODULE_PATH."/$module/controller/batch_action/{$classname}.php";
    			return;
    		} else{
    			hhClassError($classname);
 		  		return;
    		}  	
    	}	
    }
    if ($type == 'db') {
    	if (file_exists(APPLICATION_PATH."$stub/database/{$classname}.php")) {
    		require_once APPLICATION_PATH."$stub/database/{$classname}.php";
        	return;
    	} else {
    		if (file_exists(MODULE_PATH."/$module/database/{$classname}.php")) {
    			require_once MODULE_PATH."/$module/database/{$classname}.php";
    		} else{
    			hhClassError($classname);
 		  		return;
    		}  	
    	}
    }
    if ($type == 'class') {
    	if (file_exists(APPLICATION_PATH."$stub/classes/{$classname}.php")) {
    		require_once APPLICATION_PATH."$stub/classes/{$classname}.php";
        	return;
    	} else {
    		if (file_exists(MODULE_PATH."/$module/classes/{$classname}.php")) {
	    		require_once MODULE_PATH."/$module/classes/{$classname}.php";
    			return;
    		} else {
    			hhClassError($classname);
    			return;
    		}	
    	}
    }
}

function autoloadSystem($classname)
{
	$classParts = explode('_', $classname);
	$sys    = array_shift($classParts);
	$type   = array_shift($classParts);
	$last   = array_pop($classParts);
	if ($last == 'bean') $type = 'model';
	if ($last == 'action') $type = 'action';
	if ($last == 'dao')  $type = 'db';
	if ($last == 'delegate')  $type = 'db';

	if ($type == 'model') {
		require_once SYSTEM_PATH."/model/{$classname}.php";
		return;
	}
	if ($type == 'action') {
		require_once SYSTEM_PATH."/controller/{$classname}.php";
		return;
	}
	if ($type == 'db') {
		require_once SYSTEM_PATH."/database/{$classname}.php";
		return;
	}
	// anything left over will be in the classes directory
	require_once SYSTEM_PATH."/classes/{$classname}.php";

}

function autoloadExtention($classname) 
{
    $classParts = explode('_', $classname);
    $hh     = array_shift($classParts);  //indicates the framework portion of the extension
    $extension = array_shift($classParts);  //the extension name
    $type   = array_shift($classParts);  //the type of extension
    $last   = array_pop($classParts);    //the type may be indicated by the suffix
    if ($last == 'bean') $type = 'model';
    if ($last == 'dao')  $type = 'db';
    if ($last == 'delegate')  $type = 'db';
        
    if ($type == 'model') {
        require_once EXTENSION_PATH."/extensions/$extension/model/{$classname}.php";
        return;
    }
    if ($type == 'post') {
        require_once EXTENSION_PATH."/extensions/$extension/controller/post_action/{$classname}.php";
        return;
    }
    if ($type == 'nav') {
        require_once EXTENSION_PATH."/extensions/$extension/controller/navigation_action/{$classname}.php";
        return;
    }
    if ($type == 'ajax') {
        require_once EXTENSION_PATH."/extensions/$extension/controller/ajax_action/{$classname}.php";
        return;
    }
    if ($type == 'db') {
        require_once EXTENSION_PATH."/extensions/$extension/database/{$classname}.php";
        return;
    }
    // anything left over will be in the classes directory
    require_once EXTENSION."/extensions/$extension/classes/{$classname}.php";
}
spl_autoload_register ('hh_sys_autoload');

function hhClassError($classname) {
try {
	throw new Exception('Module Not Found '.$classname);
} catch (Exception $e) {
	$feedback = new sys_feedback_bean();
	$feedback->setMessage('Class Loader Error', $e->getMessage());
	$debug = callStack($e->getTrace());
	$feedback->setDebug($debug);
	$feedback->setOriginalUrl($_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']);			
	@$feedback->setReferrer ( $_SERVER ['HTTP_REFERER'] );
	$registry = sys_registry::getInstance();
	$registry->setEntry('core','feedback', $feedback);
}
}	


